package org.eclipse.viatra.cep.core.engine.runtime.util;

import org.eclipse.incquery.runtime.api.IMatchProcessor;
import org.eclipse.viatra.cep.core.engine.runtime.EnabledTransitionMatch;
import org.eclipse.viatra.cep.core.metamodels.automaton.EventToken;
import org.eclipse.viatra.cep.core.metamodels.automaton.TypedTransition;
import org.eclipse.viatra.cep.core.metamodels.events.Event;
import org.eclipse.viatra.cep.core.metamodels.events.EventPattern;

/**
 * A match processor tailored for the org.eclipse.viatra.cep.core.engine.runtime.enabledTransition pattern.
 * 
 * Clients should derive an (anonymous) class that implements the abstract process().
 * 
 */
@SuppressWarnings("all")
public abstract class EnabledTransitionProcessor implements IMatchProcessor<EnabledTransitionMatch> {
  /**
   * Defines the action that is to be executed on each match.
   * @param pTransition the value of pattern parameter transition in the currently processed match
   * @param pEventToken the value of pattern parameter eventToken in the currently processed match
   * @param pEvent the value of pattern parameter event in the currently processed match
   * @param pEventPattern the value of pattern parameter eventPattern in the currently processed match
   * 
   */
  public abstract void process(final TypedTransition pTransition, final EventToken pEventToken, final Event pEvent, final EventPattern pEventPattern);
  
  @Override
  public void process(final EnabledTransitionMatch match) {
    process(match.getTransition(), match.getEventToken(), match.getEvent(), match.getEventPattern());
  }
}
