package org.eclipse.viatra.cep.core.engine.runtime.util;

import com.google.common.collect.Sets;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.api.impl.BaseGeneratedEMFPQuery;
import org.eclipse.incquery.runtime.api.impl.BaseGeneratedEMFQuerySpecification;
import org.eclipse.incquery.runtime.emf.types.EClassTransitiveInstancesKey;
import org.eclipse.incquery.runtime.emf.types.EStructuralFeatureInstancesKey;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.incquery.runtime.matchers.psystem.PBody;
import org.eclipse.incquery.runtime.matchers.psystem.PVariable;
import org.eclipse.incquery.runtime.matchers.psystem.basicdeferred.Equality;
import org.eclipse.incquery.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.incquery.runtime.matchers.psystem.basicdeferred.NegativePatternCall;
import org.eclipse.incquery.runtime.matchers.psystem.basicenumerables.PositivePatternCall;
import org.eclipse.incquery.runtime.matchers.psystem.basicenumerables.TypeConstraint;
import org.eclipse.incquery.runtime.matchers.psystem.queries.PParameter;
import org.eclipse.incquery.runtime.matchers.psystem.queries.QueryInitializationException;
import org.eclipse.incquery.runtime.matchers.tuple.FlatTuple;
import org.eclipse.viatra.cep.core.engine.runtime.EnabledTransitionMatch;
import org.eclipse.viatra.cep.core.engine.runtime.EnabledTransitionMatcher;
import org.eclipse.viatra.cep.core.engine.runtime.util.EventProcessedByEventTokenQuerySpecification;
import org.eclipse.viatra.cep.core.engine.runtime.util.LatestEventQuerySpecification;

/**
 * A pattern-specific query specification that can instantiate EnabledTransitionMatcher in a type-safe way.
 * 
 * @see EnabledTransitionMatcher
 * @see EnabledTransitionMatch
 * 
 */
@SuppressWarnings("all")
public final class EnabledTransitionQuerySpecification extends BaseGeneratedEMFQuerySpecification<EnabledTransitionMatcher> {
  private EnabledTransitionQuerySpecification() {
    super(GeneratedPQuery.INSTANCE);
  }
  
  /**
   * @return the singleton instance of the query specification
   * @throws IncQueryException if the pattern definition could not be loaded
   * 
   */
  public static EnabledTransitionQuerySpecification instance() throws IncQueryException {
    try{
    	return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
    	throw processInitializerError(err);
    }
  }
  
  @Override
  protected EnabledTransitionMatcher instantiate(final IncQueryEngine engine) throws IncQueryException {
    return EnabledTransitionMatcher.on(engine);
  }
  
  @Override
  public EnabledTransitionMatch newEmptyMatch() {
    return EnabledTransitionMatch.newEmptyMatch();
  }
  
  @Override
  public EnabledTransitionMatch newMatch(final Object... parameters) {
    return EnabledTransitionMatch.newMatch((org.eclipse.viatra.cep.core.metamodels.automaton.TypedTransition) parameters[0], (org.eclipse.viatra.cep.core.metamodels.automaton.EventToken) parameters[1], (org.eclipse.viatra.cep.core.metamodels.events.Event) parameters[2], (org.eclipse.viatra.cep.core.metamodels.events.EventPattern) parameters[3]);
  }
  
  private static class LazyHolder {
    private final static EnabledTransitionQuerySpecification INSTANCE = make();
    
    public static EnabledTransitionQuerySpecification make() {
      return new EnabledTransitionQuerySpecification();					
    }
  }
  
  private static class GeneratedPQuery extends BaseGeneratedEMFPQuery {
    private final static EnabledTransitionQuerySpecification.GeneratedPQuery INSTANCE = new GeneratedPQuery();
    
    @Override
    public String getFullyQualifiedName() {
      return "org.eclipse.viatra.cep.core.engine.runtime.enabledTransition";
    }
    
    @Override
    public List<String> getParameterNames() {
      return Arrays.asList("transition","eventToken","event","eventPattern");
    }
    
    @Override
    public List<PParameter> getParameters() {
      return Arrays.asList(new PParameter("transition", "org.eclipse.viatra.cep.core.metamodels.automaton.TypedTransition"),new PParameter("eventToken", "org.eclipse.viatra.cep.core.metamodels.automaton.EventToken"),new PParameter("event", "org.eclipse.viatra.cep.core.metamodels.events.Event"),new PParameter("eventPattern", "org.eclipse.viatra.cep.core.metamodels.events.EventPattern"));
    }
    
    @Override
    public Set<PBody> doGetContainedBodies() throws QueryInitializationException {
      Set<PBody> bodies = Sets.newLinkedHashSet();
      try {
      {
      	PBody body = new PBody(this);
      	PVariable var_transition = body.getOrCreateVariableByName("transition");
      	PVariable var_eventToken = body.getOrCreateVariableByName("eventToken");
      	PVariable var_event = body.getOrCreateVariableByName("event");
      	PVariable var_eventPattern = body.getOrCreateVariableByName("eventPattern");
      	PVariable var_eventType = body.getOrCreateVariableByName("eventType");
      	PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
      	PVariable var__virtual_1_ = body.getOrCreateVariableByName(".virtual{1}");
      	PVariable var__virtual_2_ = body.getOrCreateVariableByName(".virtual{2}");
      	PVariable var__virtual_3_ = body.getOrCreateVariableByName(".virtual{3}");
      	PVariable var__virtual_4_ = body.getOrCreateVariableByName(".virtual{4}");
      	PVariable var__virtual_5_ = body.getOrCreateVariableByName(".virtual{5}");
      	PVariable var_automaton = body.getOrCreateVariableByName("automaton");
      	PVariable var_state = body.getOrCreateVariableByName("state");
      	PVariable var__virtual_6_ = body.getOrCreateVariableByName(".virtual{6}");
      	PVariable var__virtual_7_ = body.getOrCreateVariableByName(".virtual{7}");
      	PVariable var__virtual_8_ = body.getOrCreateVariableByName(".virtual{8}");
      	body.setExportedParameters(Arrays.<ExportedParameter>asList(
      		new ExportedParameter(body, var_transition, "transition"),
      				
      		new ExportedParameter(body, var_eventToken, "eventToken"),
      				
      		new ExportedParameter(body, var_event, "event"),
      				
      		new ExportedParameter(body, var_eventPattern, "eventPattern")
      	));
      	new TypeConstraint(body, new FlatTuple(var_transition), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "TypedTransition")));
      	new TypeConstraint(body, new FlatTuple(var_eventToken), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "EventToken")));
      	new TypeConstraint(body, new FlatTuple(var_event), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("cep.meta", "Event")));
      	new TypeConstraint(body, new FlatTuple(var_eventPattern), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("cep.meta", "EventPattern")));
      	new PositivePatternCall(body, new FlatTuple(var_event), LatestEventQuerySpecification.instance().getInternalQueryRepresentation());
      	new TypeConstraint(body, new FlatTuple(var_event), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("cep.meta", "Event")));
      	new TypeConstraint(body, new FlatTuple(var_event, var__virtual_0_), new EStructuralFeatureInstancesKey(getFeatureLiteral("cep.meta", "Event", "type")));
      	new Equality(body, var__virtual_0_, var_eventType);
      	new TypeConstraint(body, new FlatTuple(var_transition), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "TypedTransition")));
      	new TypeConstraint(body, new FlatTuple(var_transition, var__virtual_1_), new EStructuralFeatureInstancesKey(getFeatureLiteral("automaton.meta", "TypedTransition", "guard")));
      	new TypeConstraint(body, new FlatTuple(var__virtual_1_, var__virtual_2_), new EStructuralFeatureInstancesKey(getFeatureLiteral("automaton.meta", "Guard", "eventType")));
      	new TypeConstraint(body, new FlatTuple(var__virtual_2_, var__virtual_3_), new EStructuralFeatureInstancesKey(getFeatureLiteral("cep.meta", "AtomicEventPattern", "type")));
      	new Equality(body, var__virtual_3_, var_eventType);
      	new TypeConstraint(body, new FlatTuple(var_transition), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "TypedTransition")));
      	new TypeConstraint(body, new FlatTuple(var_transition, var__virtual_4_), new EStructuralFeatureInstancesKey(getFeatureLiteral("automaton.meta", "Transition", "preState")));
      	new TypeConstraint(body, new FlatTuple(var__virtual_4_, var__virtual_5_), new EStructuralFeatureInstancesKey(getFeatureLiteral("automaton.meta", "State", "eventTokens")));
      	new Equality(body, var__virtual_5_, var_eventToken);
      	new NegativePatternCall(body, new FlatTuple(var_event, var_eventToken), EventProcessedByEventTokenQuerySpecification.instance().getInternalQueryRepresentation());
      	new TypeConstraint(body, new FlatTuple(var_automaton), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "Automaton")));
      	new TypeConstraint(body, new FlatTuple(var_automaton, var__virtual_6_), new EStructuralFeatureInstancesKey(getFeatureLiteral("automaton.meta", "Automaton", "states")));
      	new Equality(body, var__virtual_6_, var_state);
      	new TypeConstraint(body, new FlatTuple(var_state), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "State")));
      	new TypeConstraint(body, new FlatTuple(var_state, var__virtual_7_), new EStructuralFeatureInstancesKey(getFeatureLiteral("automaton.meta", "State", "outTransitions")));
      	new Equality(body, var__virtual_7_, var_transition);
      	new TypeConstraint(body, new FlatTuple(var_automaton), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "Automaton")));
      	new TypeConstraint(body, new FlatTuple(var_automaton, var__virtual_8_), new EStructuralFeatureInstancesKey(getFeatureLiteral("automaton.meta", "Automaton", "eventPattern")));
      	new Equality(body, var__virtual_8_, var_eventPattern);
      	bodies.add(body);
      }
      	// to silence compiler error
      	if (false) throw new IncQueryException("Never", "happens");
      } catch (IncQueryException ex) {
      	throw processDependencyException(ex);
      }
      return bodies;
    }
  }
}
