package org.eclipse.viatra.cep.core.engine.runtime.util;

import com.google.common.collect.Sets;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.api.impl.BaseGeneratedEMFPQuery;
import org.eclipse.incquery.runtime.api.impl.BaseGeneratedEMFQuerySpecification;
import org.eclipse.incquery.runtime.emf.types.EClassTransitiveInstancesKey;
import org.eclipse.incquery.runtime.emf.types.EStructuralFeatureInstancesKey;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.incquery.runtime.matchers.psystem.PBody;
import org.eclipse.incquery.runtime.matchers.psystem.PVariable;
import org.eclipse.incquery.runtime.matchers.psystem.basicdeferred.Equality;
import org.eclipse.incquery.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.incquery.runtime.matchers.psystem.basicdeferred.NegativePatternCall;
import org.eclipse.incquery.runtime.matchers.psystem.basicenumerables.TypeConstraint;
import org.eclipse.incquery.runtime.matchers.psystem.queries.PParameter;
import org.eclipse.incquery.runtime.matchers.psystem.queries.QueryInitializationException;
import org.eclipse.incquery.runtime.matchers.tuple.FlatTuple;
import org.eclipse.viatra.cep.core.engine.runtime.PartiallyMatchedEventPatternMatch;
import org.eclipse.viatra.cep.core.engine.runtime.PartiallyMatchedEventPatternMatcher;
import org.eclipse.viatra.cep.core.engine.runtime.util.InitStateQuerySpecification;

/**
 * A pattern-specific query specification that can instantiate PartiallyMatchedEventPatternMatcher in a type-safe way.
 * 
 * @see PartiallyMatchedEventPatternMatcher
 * @see PartiallyMatchedEventPatternMatch
 * 
 */
@SuppressWarnings("all")
public final class PartiallyMatchedEventPatternQuerySpecification extends BaseGeneratedEMFQuerySpecification<PartiallyMatchedEventPatternMatcher> {
  private PartiallyMatchedEventPatternQuerySpecification() {
    super(GeneratedPQuery.INSTANCE);
  }
  
  /**
   * @return the singleton instance of the query specification
   * @throws IncQueryException if the pattern definition could not be loaded
   * 
   */
  public static PartiallyMatchedEventPatternQuerySpecification instance() throws IncQueryException {
    try{
    	return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
    	throw processInitializerError(err);
    }
  }
  
  @Override
  protected PartiallyMatchedEventPatternMatcher instantiate(final IncQueryEngine engine) throws IncQueryException {
    return PartiallyMatchedEventPatternMatcher.on(engine);
  }
  
  @Override
  public PartiallyMatchedEventPatternMatch newEmptyMatch() {
    return PartiallyMatchedEventPatternMatch.newEmptyMatch();
  }
  
  @Override
  public PartiallyMatchedEventPatternMatch newMatch(final Object... parameters) {
    return PartiallyMatchedEventPatternMatch.newMatch((org.eclipse.viatra.cep.core.metamodels.automaton.Automaton) parameters[0], (org.eclipse.viatra.cep.core.metamodels.automaton.EventToken) parameters[1], (org.eclipse.viatra.cep.core.metamodels.automaton.State) parameters[2]);
  }
  
  private static class LazyHolder {
    private final static PartiallyMatchedEventPatternQuerySpecification INSTANCE = make();
    
    public static PartiallyMatchedEventPatternQuerySpecification make() {
      return new PartiallyMatchedEventPatternQuerySpecification();					
    }
  }
  
  private static class GeneratedPQuery extends BaseGeneratedEMFPQuery {
    private final static PartiallyMatchedEventPatternQuerySpecification.GeneratedPQuery INSTANCE = new GeneratedPQuery();
    
    @Override
    public String getFullyQualifiedName() {
      return "org.eclipse.viatra.cep.core.engine.runtime.partiallyMatchedEventPattern";
    }
    
    @Override
    public List<String> getParameterNames() {
      return Arrays.asList("automaton","eventToken","state");
    }
    
    @Override
    public List<PParameter> getParameters() {
      return Arrays.asList(new PParameter("automaton", "org.eclipse.viatra.cep.core.metamodels.automaton.Automaton"),new PParameter("eventToken", "org.eclipse.viatra.cep.core.metamodels.automaton.EventToken"),new PParameter("state", "org.eclipse.viatra.cep.core.metamodels.automaton.State"));
    }
    
    @Override
    public Set<PBody> doGetContainedBodies() throws QueryInitializationException {
      Set<PBody> bodies = Sets.newLinkedHashSet();
      try {
      {
      	PBody body = new PBody(this);
      	PVariable var_automaton = body.getOrCreateVariableByName("automaton");
      	PVariable var_eventToken = body.getOrCreateVariableByName("eventToken");
      	PVariable var_state = body.getOrCreateVariableByName("state");
      	PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
      	PVariable var__virtual_1_ = body.getOrCreateVariableByName(".virtual{1}");
      	body.setExportedParameters(Arrays.<ExportedParameter>asList(
      		new ExportedParameter(body, var_automaton, "automaton"),
      				
      		new ExportedParameter(body, var_eventToken, "eventToken"),
      				
      		new ExportedParameter(body, var_state, "state")
      	));
      	new TypeConstraint(body, new FlatTuple(var_automaton), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "Automaton")));
      	new TypeConstraint(body, new FlatTuple(var_eventToken), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "EventToken")));
      	new TypeConstraint(body, new FlatTuple(var_state), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "State")));
      	new TypeConstraint(body, new FlatTuple(var_automaton), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "Automaton")));
      	new TypeConstraint(body, new FlatTuple(var_automaton, var__virtual_0_), new EStructuralFeatureInstancesKey(getFeatureLiteral("automaton.meta", "Automaton", "eventTokens")));
      	new Equality(body, var__virtual_0_, var_eventToken);
      	new TypeConstraint(body, new FlatTuple(var_state), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("automaton.meta", "State")));
      	new TypeConstraint(body, new FlatTuple(var_state, var__virtual_1_), new EStructuralFeatureInstancesKey(getFeatureLiteral("automaton.meta", "State", "eventTokens")));
      	new Equality(body, var__virtual_1_, var_eventToken);
      	new NegativePatternCall(body, new FlatTuple(var_automaton, var_state), InitStateQuerySpecification.instance().getInternalQueryRepresentation());
      	bodies.add(body);
      }
      	// to silence compiler error
      	if (false) throw new IncQueryException("Never", "happens");
      } catch (IncQueryException ex) {
      	throw processDependencyException(ex);
      }
      return bodies;
    }
  }
}
